/**clutter_jammo.c is part of JamMo.
License: GPLv2, read more from COPYING

This is 'default' Graphical user interface for JamMo. It implements 
all functions in gui.h. This is base file for whole clutter-gui.

Read more about clutter: http://clutter-project.org/
*/
//This is because of getline
#define _GNU_SOURCE

#include <stdlib.h>
#include <gtk/gtk.h>
#include <clutter/clutter.h>
#include <glib-object.h>
#include <string.h>
#include <pthread.h>


#include "../chum/chum.h"
#include "../chum/gui.h"
#include "../meam/meam.h"  //FIXME. get rid of this.

#include "clutter_jammo.h"
#include "config.h"
#include "object.h"
#include "button.h"
#include "jammo-texture.h"
#include "json_helper.h" //there are generate_json_for_theme

//Games
#include "composing_game.h"
#include "singing_game.h"
#include "instrument_testing.h" //For testing virtual instruments (and slider)

#include "midi_editor.h" //This will go under advanced composing game

ClutterColor trans_color = { 0, 0, 0, 0x00 };
ClutterColor dark_color =  { 0, 0, 0, 0xdd };
ClutterColor white_color = { 0, 0, 0, 0xbb };
ClutterColor blue_color =  { 0, 0, 0xff, 0xff };

ClutterColor *get_transparency_color(){ return &trans_color; }
ClutterColor *get_white_color(){        return &white_color; }
ClutterColor *get_dark_color(){         return &dark_color;  }
ClutterColor *get_blue_color(){         return &blue_color;  }


ClutterActor* stage;

ClutterActor* main_getStage(){
  return stage;
}

//////For welcome screen, animating door opening
#define angle_of_opened_door 110.0
gdouble door_rotating=0.0;
static gboolean rotate_actor (gpointer data){
  door_rotating-=0.8;
  clutter_actor_set_rotation(CLUTTER_ACTOR(data),CLUTTER_Y_AXIS,door_rotating,0,0,0);

  if (door_rotating>-angle_of_opened_door)
    return TRUE;

  //just be sure, that there aren't any sounds when going next step.
  chum_stop_one_sample();
  chum_go_next();
  return FALSE;
}

static void welcome_end (JammoTexture *actor, ClutterEvent *event, gpointer data){
  //if pressed already
  if (door_rotating<0.0)
    {
    door_rotating=-angle_of_opened_door;
    return;
    }

  g_timeout_add (50,rotate_actor,actor);
  chum_play_one_sample(g_strdup_printf("%s/door_opening.wav",DATA_DIR));
}


static void show_welcome(){
  ClutterActor* stage =main_getStage();

  ClutterActor *s1 = config_get_configured_actor(NULL,"welcome");
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), s1);
  clutter_actor_show (s1);
  clutter_actor_set_reactive(s1, TRUE);
  clutter_actor_set_name(s1, "welcome");

  s1 = config_get_configured_actor(NULL,"door");
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), s1);

  clutter_actor_show (s1);
  clutter_actor_set_reactive(s1, TRUE);
  clutter_actor_set_name(s1, "door");
  g_signal_connect (s1, "button-release-event", G_CALLBACK (welcome_end),NULL);
}



static void compose_clicked(){
chum_go_screen(SCREEN_THEME_SELECTION);
}

static void singing_clicked(){
chum_go_screen(SCREEN_SONG_SELECTION);
}


void show_game_selection() {
  ClutterActor* stage =main_getStage();

  ClutterActor *s1 = config_get_configured_actor(NULL,"house");
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), s1);
  clutter_actor_show (s1);
  clutter_actor_set_name(s1, "house");

  s1 = config_get_configured_actor(NULL,"select_singing");
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), s1);
  clutter_actor_show (s1);
  clutter_actor_set_reactive(s1, TRUE);
  clutter_actor_set_name(s1, "select_singing");
  g_signal_connect (s1, "button-release-event", G_CALLBACK (singing_clicked),NULL);

  s1 = config_get_configured_actor(NULL,"select_composing");
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), s1);
  clutter_actor_show (s1);
  clutter_actor_set_reactive(s1, TRUE);
  clutter_actor_set_name(s1, "select_composing");
  g_signal_connect (s1, "button-release-event", G_CALLBACK (compose_clicked),NULL);

  s1 = config_get_configured_actor(NULL,"select_ud");
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), s1);
  clutter_actor_show (s1);
  clutter_actor_set_reactive(s1, TRUE);
  clutter_actor_set_name(s1, "select_ud");
  g_signal_connect (s1, "button-release-event", G_CALLBACK (start_test_instrument),GINT_TO_POINTER(2));

  s1 = config_get_configured_actor(NULL,"select_drum");
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), s1);
  clutter_actor_show (s1);
  clutter_actor_set_reactive(s1, TRUE);
  clutter_actor_set_name(s1, "select_drum");
  g_signal_connect (s1, "button-release-event", G_CALLBACK (start_test_instrument),GINT_TO_POINTER(1));

  s1 = config_get_configured_actor(NULL,"select_flute");
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), s1);
  clutter_actor_show (s1);
  clutter_actor_set_reactive(s1, TRUE);
  clutter_actor_set_name(s1, "select_flute");
  g_signal_connect (s1, "button-release-event", G_CALLBACK (start_test_instrument),GINT_TO_POINTER(0));

  s1 = config_get_configured_actor(NULL,"select_loading");
  clutter_container_add_actor (CLUTTER_CONTAINER (stage), s1);
  clutter_actor_set_size(s1,60,60);
  clutter_actor_set_position(s1,10,400);
  clutter_actor_show (s1);
  clutter_actor_set_reactive(s1, TRUE);
  clutter_actor_set_name(s1, "house");
  g_signal_connect (s1, "button-release-event", G_CALLBACK (start_slider),NULL);
}



/////////////////////////////////////////////////
void gui_go_screen(int screen_number) {
  if (screen_number==SCREEN_WELCOME){
    show_welcome();
  } else if (screen_number==SCREEN_GAME_SELECTION){
    show_game_selection();
  } else if (screen_number==SCREEN_THEME_SELECTION){
    show_theme_selection();
  } else if (screen_number==SCREEN_COMPOSING){
    start_theme();
  } else if (screen_number==SCREEN_SONG_SELECTION){
    start_mobikid_selection();
  } else if (screen_number==SCREEN_SONG_RECORDING){
    start_mobikid_recording();
  } else if (screen_number==SCREEN_PLAY_RECORDED){
    start_mobikid_play_recorded();
  } else if (screen_number==SCREEN_MIDI_EDITOR){
    start_midi_editor();
  }

}



/*We aren't using gtk-loop/clutter_main, but pipelines loop*/
void window_closed(void) {
  printf("window is closed\n");
  g_main_loop_quit(get_mainPipeline()->loop); 
  }

GdkPixbuf *create_pixbuf(const gchar * filename)
  {
    GdkPixbuf *pixbuf;
    GError *error = NULL;
    pixbuf = gdk_pixbuf_new_from_file(filename, &error);
    if(!pixbuf) {
	fprintf(stderr, "%s\n", error->message);
	g_error_free(error);
    }

    return pixbuf;
}


/*This can handle other events too.*/
static void event_handler (ClutterActor    *actor,
          ClutterEvent    *event,
          gpointer         data)
{
  switch (event->type)
    {
    case CLUTTER_DELETE:
      window_closed();
    default:
      break;
    }
}

void gui_init() {
      int i;
      gint number_of_themes =chum_get_number_of_themes();
      for (i=1;i<number_of_themes+1;i++){
        //printf("name of theme: %s\n",chum_get_theme_folder(i));
        generate_json_for_theme(chum_get_theme_folder(i));
      }

      clutter_init(NULL,NULL);

      //stage is global
      stage = clutter_stage_get_default();
      clutter_stage_set_title(CLUTTER_STAGE(stage),"JamMo");
      //clutter_stage_set_user_resizable (CLUTTER_STAGE(stage),TRUE);
      clutter_actor_set_size (stage, WIDTH_OF_WINDOW,HEIGHT_OF_WINDOW);

      g_signal_connect (stage, "event", G_CALLBACK (event_handler), "stage");

      #ifdef N900
      clutter_stage_set_fullscreen(CLUTTER_STAGE(stage),TRUE);
      #endif
      clutter_actor_show_all(stage);

      pthread_t loop_thread;
      pthread_create(&loop_thread, NULL, start_reading_joystick, NULL);
}

#ifdef clutter_gtk_way
/*This is way how to use clutter-gtk. JamMo doesn't need clutter-gtk.
  Add clutter-gtk to configure.ac and debian/control if you want use this.
*/
#include <clutter-gtk/clutter-gtk.h>
void gui_init() {
      int i;
      gint number_of_themes =chum_get_number_of_themes();
      for (i=1;i<number_of_themes+1;i++){
	//printf("name of theme: %s\n",chum_get_theme_folder(i));
	generate_json_for_theme(chum_get_theme_folder(i));
      }

	GtkWidget* window;
	GtkWidget* vbox;
	GtkWidget* embed;
	
	clutter_init(NULL,NULL);
	gtk_init(NULL,NULL);
	
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
	gtk_window_set_title(GTK_WINDOW(window), "JamMo");
	gtk_window_resize( GTK_WINDOW(window), WIDTH_OF_WINDOW,HEIGHT_OF_WINDOW );
	gtk_window_set_icon(GTK_WINDOW(window), create_pixbuf("/usr/share/pixmaps/jammo.png"));

	vbox = gtk_vbox_new(FALSE, 8);
	gtk_container_add(GTK_CONTAINER(window), vbox);
	
	embed = gtk_clutter_embed_new();
	gtk_widget_set_size_request(embed, WIDTH_OF_WINDOW,HEIGHT_OF_WINDOW);
	gtk_box_pack_start(GTK_BOX(vbox), embed, FALSE, FALSE, 0);
	stage = gtk_clutter_embed_get_stage(GTK_CLUTTER_EMBED(embed));

	// Initializes main window to be drawable
	gtk_widget_set_app_paintable(window, TRUE);
	
	clutter_actor_show(stage);
	gtk_widget_show_all(window); //This causes "Unable to make the stage window 0x3c00020 the current GLX drawable", so it must be disable, if running in gdb. Window size is then wrong.
	//and it doesn't send "destroy" signal when closed.

	g_signal_connect(window, "destroy",G_CALLBACK(window_closed), NULL);

    pthread_t loop_thread;
    pthread_create(&loop_thread, NULL, start_reading_joystick, NULL);
}
#endif
